<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0">

    <xsl:output method="text"/>
    <xsl:output indent="yes" method="xml" name="xml"/>
    
    <xsl:param name="outputDir" select="'../../schemas/'"/>
    
    <xsl:variable name="vocabInteractions" select="concat($outputDir,'../vocab/2.16.840.1.113883.1.6.xml')"/>
    
    <!--store variables-->
    <xsl:variable name="interactionName" select="//interaction[1]/@name"/>
    <xsl:variable name="version" select="//interaction[1]/@version"/>
    <xsl:variable name="interactionNameVersion" select="concat($interactionName, if (string-length($version)>0) then $version else '')"/>
    
    <xsl:variable name="transmissionWrapper" select="//transwrap/@refIdentifier"/>
    <xsl:variable name="controlActWrapper" select="//cactwrap/@refIdentifier"/>
    <xsl:variable name="payload" select="(//msgtype)[1]/@refIdentifier"/>
    <xsl:variable name="queryByParameterPayload" select="(//msgtype)[2]/@refIdentifier"/>
    <xsl:variable name="msgTypeCount" select="count(//msgtype)"/>
    
    <xsl:variable name="outputFilename" select="concat($outputDir, $interactionNameVersion, '.xsd')"/>
    
    <xsl:template match="/">
        <!-- Output to log  -->
        <xsl:message select="concat(format-dateTime(current-dateTime(), '[Y]-[M,2]-[D,2] [H,2]:[m,2]:[s,2]') , '. Creating ', $outputFilename, '&#10;')"/>
        <xsl:for-each select="//*[local-name()='transwrap' or local-name()='cactwrap' or local-name()='msgtype']">
            <xsl:call-template name="checkIfSchemaExists">
                <xsl:with-param name="schemaName" select="concat(@refIdentifier,'.xsd')"/>
            </xsl:call-template>
        </xsl:for-each>
        <!-- Output to log  -->

        <!-- The interaction output file starts here -->
        <xsl:result-document href="{$outputFilename}" format="xml" indent="yes" standalone="no">
            
            <xsl:call-template name="outputDescriptionComment"/>
            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns="urn:hl7-org:v3" targetNamespace="urn:hl7-org:v3" elementFormDefault="qualified">
                <xs:include schemaLocation="{concat($transmissionWrapper,'.xsd')}"/>
                <xsl:if test="string-length($controlActWrapper)>0">
                    <xs:include schemaLocation="{concat($controlActWrapper,'.xsd')}"/>
                    <xsl:choose>
                        <xsl:when test="string-length($payload)>0">
                            <xs:include schemaLocation="{concat($payload,'.xsd')}"/>
                            <xsl:if test="string-length($queryByParameterPayload)>0">
                                <xs:include schemaLocation="{concat($queryByParameterPayload,'.xsd')}"/>
                            </xsl:if>
                        </xsl:when>
                        <xsl:otherwise>
                            <xs:group name="Payload"><xs:sequence/></xs:group>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:if>
                
                <xs:element name="{$interactionNameVersion}" type="{concat($transmissionWrapper,'.Message')}"/>
                
                <xsl:for-each select="  //Payload[@name][@type] | 
                                        //QueryByParameterPayload[@name][@type] |
                                        //QueryByParameterParameterList[@name][@type]">
                    <xs:group name="{name(.)}">
                        <xs:sequence>
                            <xs:element>
                                <xsl:copy-of select="@name | @type" copy-namespaces="no"/>
                            </xs:element>
                        </xs:sequence>
                    </xs:group>
                </xsl:for-each>
            </xs:schema>
            
        </xsl:result-document>
    </xsl:template>
    
    <xsl:template name="outputDescriptionComment" xmlns:hl7="urn:hl7-org:v3">
        <xsl:variable name="initialComment">
            <xsl:if test="doc-available($vocabInteractions)">
                <xsl:variable name="displayName"
                    select="document($vocabInteractions)/*/hl7:code[@code=$interactionNameVersion][@codeSystem='2.16.840.1.113883.1.6']/@displayName"/>
                <xsl:if test="string-length($displayName)">
                    <xsl:value-of select="concat(' ' , $interactionNameVersion, ' - ' , $displayName , ' ')"/>
                </xsl:if>
            </xsl:if>
        </xsl:variable>
        <xsl:text>&#10;</xsl:text>
        <xsl:comment>
            <xsl:choose>
                <xsl:when test="string-length($initialComment)>0">
                    <xsl:value-of select="$initialComment"/>
                    <xsl:message>Interaction name <xsl:value-of select="$initialComment"/></xsl:message>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>Interaction not available in vocab at generation time</xsl:text>
                    <xsl:message>Interaction not available in vocab at generation time</xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:comment>
        <xsl:comment>
            <xsl:text>Note: this file was generated. DO NOT EDIT HERE.</xsl:text>
        </xsl:comment>
        <xsl:text>&#10;</xsl:text>
    </xsl:template>
    
    <xsl:template name="checkIfSchemaExists">
        <xsl:param name="schemaName"/>
        
        <xsl:if test="not(doc-available(concat($outputDir,$schemaName)))">
            <xsl:message terminate="no" select="concat('[ERROR] Interaction ', $interactionNameVersion, ' verwijst naar niet-bestaand XML Schema ',concat($outputDir,$schemaName))"/>
            <xsl:comment select="concat('[ERROR] Interaction ', $interactionNameVersion, ' verwijst naar niet-bestaand XML Schema ',concat($outputDir,$schemaName))"/>
            <xsl:text>&#10;</xsl:text>            
        </xsl:if>
    </xsl:template>
</xsl:stylesheet>
