<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl" xmlns:hl7="urn:hl7-org:v3" exclude-result-prefixes="#all" version="2.0">
    <xd:doc scope="stylesheet">
        <xd:desc>
            <xd:p><xd:b>Created on:</xd:b> Jul 7, 2011</xd:p>
            <xd:p><xd:b>Author:</xd:b> ahenket</xd:p>
            <xd:p><xd:b>Based on:</xd:b> work by gboers</xd:p>
            <xd:p><xd:b>Purpose:</xd:b> reads any kind of XML Schema and converts them to a new schema. It handles three types: <xd:ul>
                    <xd:li><xd:i>interaction</xd:i> file pattern AAAA_INnnnnnn</xd:li>
                    <xd:li><xd:i>messagetype</xd:i> file pattern AAAA_MTnnnnnn</xd:li>
                    <xd:li><xd:i>unknown</xd:i> any other file pattern.</xd:li>
                </xd:ul>
                <xd:p><xd:b>interaction pattern</xd:b> is handled in a special manner. The following is the maximum of steps that may occur.
                    Interactions like MCCI_IN000002 have no ControlAct, and other do not have Payload, QueryByParameter or RegistrationProcess: <xd:ul>
                        <xd:li>Add interaction prefix to element type declaration, e.g. MFMT_IN002101.MCCI_MT000100.Message</xd:li>
                        <xd:li>Add new transmission wrapper complexType with the name from the previous step, containing the actual complexType from
                            the transmission wrapper, e.g. MCCI_MT000100.Message. <xd:ul>
                                <xd:li>Replace xs:group ref='ControlAct' in this transmission wrapper complexType with the contents of that particular
                                    group. The correct declaration is pulled from the ControlAct that is included in the interaction file by checking
                                    each include for that particular group. Normally this is an xs:sequence containing one element. The type for that
                                    element is prefixed with the interaction, e.g. MFMT_IN002101.MFMI_MT700702.ControlActProcess</xd:li>
                            </xd:ul></xd:li>
                        <xd:li>Add new control act complexType with the name from the previous step, containing the actual complexType from the
                            control act, e.g. MFMI_MT700702.ControlActProcess. <xd:ul>
                                <xd:li>Replace subject element type with an interaction prefixed version, e.g.
                                    MFMT_IN002101.MFMI_MT700702.Subject</xd:li>
                                <xd:li>Replace xs:group ref='QueryByParameterPayload' with actual contents of that group. The correct declaration is
                                    pulled from the QueryByParameter file that is included in the interaction file by checking each include for that
                                    particular group.</xd:li>
                            </xd:ul></xd:li>
                        <xd:li>If ControlAct file has a complexType with a name ending in .RegistrationProcess, add new complexType containing the
                            actual complexType from the ControlAct, but with an interaction prefixed name.<xd:ul>
                                <xd:li>Replace both subject1 and subject2 with an interaction prefixed type, e.g.
                                    MFMT_IN002101.MFMI_MT700702.Subject4</xd:li>
                            </xd:ul></xd:li>
                        <xd:li>Add new complexTypes for both subject1 and subject2 elements by pulling those from the actual ControlAct. In both of
                            these complexTypes, replace xs:group ref='Payload' with the actual contents of that group</xd:li>
                    </xd:ul></xd:p>
                <xd:p><xd:b>messagetype pattern</xd:b> is handled special. <xd:ul>
                        <xd:li>Remove any top level element</xd:li>
                        <xd:li>Remove any group with the name Message, ControlAct, Payload or QueryByParameter</xd:li>
                        <xd:li>Remove any complexType containing a group ref='Message', 'ControlAct', 'Payload' or 'QueryByParameter'</xd:li>
                        <xd:li>In registry based message typoes remove complexTypes .Subject, .RegistrationProcess, Subject4, and Subject2</xd:li>
                    </xd:ul></xd:p>
                <xd:p><xd:b>unknown</xd:b> is handled by copying as-is.</xd:p>
            </xd:p>
        </xd:desc>
    </xd:doc>

    <xsl:output indent="yes"/>
    <xsl:strip-space elements="*"/>

    <xsl:variable name="vocabInteractions" select="'../../vocab/2.16.840.1.113883.1.6.xml'"/>
    <xsl:variable name="currentFileName" select="tokenize(document-uri(.),'/')[last()]"/>
    <xsl:variable name="currentFileDir" select="replace(document-uri(.),$currentFileName,'')"/>
    <xsl:variable name="currentFileNameBase" select="tokenize($currentFileName,'\.')[last()-1]"/>
    <xsl:variable name="currentFileType">
        <xsl:choose>
            <xsl:when test="matches($currentFileNameBase,'^[A-Z]{4}_IN[0-9]{6}')">
                <xsl:text>interaction</xsl:text>
            </xsl:when>
            <xsl:when test="matches($currentFileNameBase,'^[A-Z]{4}_MT[0-9]{6}')">
                <xsl:text>messagetype</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:text>unknown</xsl:text>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <!-- Get types and references from includes and root element definition -->
    <xsl:variable name="interactionFile" select="document-uri(.)"/>
    <!-- interactionId -->
    <xsl:variable name="interactionId">
        <xsl:value-of select="/xs:schema/xs:element/@name"/>
    </xsl:variable>
    <xsl:variable name="interactionType">
        <xsl:choose>
            <xsl:when test="ends-with(/xs:schema/xs:element/@type,'.Message')">
                <xsl:value-of select="/xs:schema/xs:element/@type"/>
            </xsl:when>
            <!-- CareProvision schemas -->
            <xsl:when test="ends-with(/xs:schema/xs:element/xs:complexType/xs:complexContent/xs:extension/@base,'.Message')">
                <xsl:value-of select="/xs:schema/xs:element/xs:complexType/xs:complexContent/xs:extension/@base"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:text/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <xsl:variable name="transmissionGroupName" select="'Message'"/>
    <xsl:variable name="controlActGroupName" select="'ControlAct'"/>
    <xsl:variable name="payloadGroupName" select="'Payload'"/>
    <xsl:variable name="queryByParameterGroupName" select="'QueryByParameterPayload'"/>
    <xsl:variable name="queryByParameterParameterListGroupName" select="'QueryByParameterParameterList'"/>

    <!-- copy pi and comments -->
    <xsl:template match="processing-instruction()|comment()" mode="#all">
        <xsl:copy-of select="."/>
    </xsl:template>

    <!-- Get incoming filetype (interaction, messagetype, unknown. Special handling of interaction and messagetype.
        Copy as-is for unknown
    -->
    <xsl:template match="/">
        <xsl:choose>
            <xsl:when test="$currentFileType='interaction'">
                <xsl:message>
                    <xsl:text>    </xsl:text>
                    <xsl:value-of select="$currentFileName"/>
                    <xsl:text> heeft bekend type '</xsl:text>
                    <xsl:value-of select="$currentFileType"/>
                    <xsl:text>'...</xsl:text>
                </xsl:message>
                <xsl:apply-templates mode="interaction"/>
            </xsl:when>
            <xsl:when test="$currentFileType='messagetype'">
                <xsl:message>
                    <xsl:text>    </xsl:text>
                    <xsl:value-of select="$currentFileName"/>
                    <xsl:text> heeft bekend type '</xsl:text>
                    <xsl:value-of select="$currentFileType"/>
                    <xsl:text>'...</xsl:text>
                </xsl:message>
                <xsl:apply-templates mode="messagetype"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:message>
                    <xsl:text>    </xsl:text>
                    <xsl:value-of select="$currentFileName"/>
                    <xsl:text> heeft geen bekend type. Geen wijzigingen, alleen kopiëren...</xsl:text>
                </xsl:message>
                <xsl:apply-templates/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- Remove the payload group declaration for HL7 ping, tick tock interactions -->
    <xsl:template
        match="xs:group[@name=$transmissionGroupName or @name=$controlActGroupName or @name=$payloadGroupName or @name=$queryByParameterGroupName or @name='MCCI_MT000100']"
        mode="#all">
        <xsl:comment>
            <xsl:value-of select="@name"/>
            <xsl:text> group declaration removed</xsl:text>
        </xsl:comment>
    </xsl:template>

    <xsl:template match="xs:group[@ref=$payloadGroupName or @ref=$queryByParameterGroupName or @ref=$queryByParameterParameterListGroupName]" mode="interaction">
        <!-- get MessageType from include -->
        <xsl:variable name="payloadFile">
            <xsl:call-template name="getOtherIncludeFromInteraction">
                <xsl:with-param name="includeType" select="@ref"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="ref" select="@ref"/>
        <xsl:choose>
            <xsl:when test="document($interactionFile)/xs:schema/xs:group[@name=$ref]">
                <xsl:element name="{name(document($interactionFile)/xs:schema/xs:group[@name=$ref]/node())}">
                    <xsl:for-each select="document($interactionFile)/xs:schema/xs:group[@name=$ref]/node()/@*">
                        <xsl:attribute name="{name(.)}" select="."/>
                    </xsl:for-each>
                    <xsl:for-each select="@*">
                        <xsl:if test="not(name(.)='ref')">
                            <xsl:attribute name="{name(.)}" select="."/>
                        </xsl:if>
                    </xsl:for-each>
                    <xsl:copy-of select="document($interactionFile)/xs:schema/xs:group[@name=$ref]/node()/node()"/>
                </xsl:element>
            </xsl:when>
            <xsl:when test="doc-available($payloadFile)">
                <xsl:element name="{name(document($payloadFile)/xs:schema/xs:group[@name=$ref]/node())}">
                    <xsl:for-each select="document($payloadFile)/xs:schema/xs:group[@name=$ref]/node()/@*">
                        <xsl:attribute name="{name(.)}" select="."/>
                    </xsl:for-each>
                    <xsl:for-each select="@*">
                        <xsl:if test="not(name(.)='ref')">
                            <xsl:attribute name="{name(.)}" select="."/>
                        </xsl:if>
                    </xsl:for-each>
                    <xsl:copy-of select="document($payloadFile)/xs:schema/xs:group[@name=$ref]/node()/node()"/>
                </xsl:element>
            </xsl:when>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="xs:schema/xs:element" mode="interaction">
        <xsl:choose>
            <!-- Redefine the root element by putting the interaction name in front -->
            <xsl:when test="ends-with(@type,'.Message') and not(starts-with(@type,$interactionId))">
                <xsl:element name="xs:element">
                    <xsl:attribute name="name" select="@name"/>
                    <xsl:attribute name="type" select="concat(@name,'.',@type)"/>
                </xsl:element>
            </xsl:when>
            <!-- Redefine the root element for CareProvision schemas -->
            <xsl:when
                test="ends-with(xs:complexType/xs:complexContent/xs:extension/@base,'.Message') and
                not(starts-with(xs:complexType/xs:complexContent/xs:extension/@base,$interactionId))">
                <xsl:element name="xs:element">
                    <xsl:attribute name="name" select="@name"/>
                    <xsl:element name="xs:complexType">
                        <xsl:element name="xs:complexContent">
                            <xsl:element name="xs:extension">
                                <xsl:attribute name="base" select="concat(@name,'.',xs:complexType/xs:complexContent/xs:extension/@base)"/>
                                <xsl:apply-templates select="xs:complexType/xs:complexContent/xs:extension/node()" mode="#current"/>
                            </xsl:element>
                        </xsl:element>
                    </xsl:element>
                </xsl:element>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/>
            </xsl:otherwise>
        </xsl:choose>

        <xsl:variable name="transmissionWrapperType">
            <xsl:choose>
                <!-- Redefine the root element by putting the interaction name in front -->
                <xsl:when test="ends-with(@type,'.Message') and not(starts-with(@type,$interactionId))">
                    <xsl:value-of select="@type"/>
                </xsl:when>
                <!-- Redefine the root element for CareProvision schemas -->
                <xsl:when
                    test="ends-with(xs:complexType/xs:complexContent/xs:extension/@base,'.Message') and
                not(starts-with(xs:complexType/xs:complexContent/xs:extension/@base,$interactionId))">
                    <xsl:value-of select="xs:complexType/xs:complexContent/xs:extension/@base"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <!-- get transmissionWrapper type from include -->
        <xsl:variable name="transmissionWrapperFile">
            <xsl:call-template name="getTransmissionIncludeFromInteraction">
                <xsl:with-param name="includeType" select="$interactionType"/>
            </xsl:call-template>
        </xsl:variable>

        <!-- get controlActWrapper type from include -->
        <xsl:variable name="controlActWrapperFile">
            <xsl:call-template name="getOtherIncludeFromInteraction">
                <xsl:with-param name="includeType" select="$controlActGroupName"/>
            </xsl:call-template>
        </xsl:variable>

        <xsl:if test="$transmissionWrapperType!='' and doc-available($transmissionWrapperFile)">
            <xsl:call-template name="outputTransmissionWrapperComplexType">
                <xsl:with-param name="transmissionWrapperFile" select="$transmissionWrapperFile"/>
                <xsl:with-param name="controlActWrapperFile" select="$controlActWrapperFile"/>
            </xsl:call-template>
        </xsl:if>

        <!--
            Create controlAct type definition.
            This definition replaces the controlActProcess definition in the controlAct wrappers.
            Hanldes the following exceptions:
            - different naming of the subject reference
            - different naming of the queryByParameter type definition
            - differences in cardinalities
        -->
        <xsl:if test="$controlActWrapperFile!='' and doc-available($controlActWrapperFile)">
            <xsl:call-template name="outputControlActWrapperComplexType">
                <xsl:with-param name="transmissionWrapperFile" select="$transmissionWrapperFile"/>
                <xsl:with-param name="controlActWrapperFile" select="$controlActWrapperFile"/>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>

    <!-- (messagetype) Remove any element declarations immediately under the xs:schema element, we only need 1 root element per interaction -->
    <xsl:template match="xs:schema/xs:element" mode="messagetype">
        <xsl:comment>
            <xsl:value-of select="@name"/>
            <xsl:text> </xsl:text>
            <xsl:value-of select="name(.)"/>
            <xsl:text> declaration removed</xsl:text>
        </xsl:comment>
    </xsl:template>

    <!-- (messagetype) Remove any complexType declarations that matches with the xs:group ref='Message' or ref='ControlAct' or
        where complexType declaration contains a group ref='ControlAct' or ref='Payload' or ref='QueryByParameterPayload'.
        
        The purpose is to eliminate any complexTypes with references to the 4 anonymous groups 'Message', 'ControlAct', 
        'Payload', and 'QueryByParameterPayload'
    -->
    <xsl:template
        match="xs:schema/xs:complexType[@name=../xs:group[@name=$transmissionGroupName or @name=$controlActGroupName]//xs:element/@type or
        xs:sequence/xs:group[@ref=$controlActGroupName or @ref=$payloadGroupName or @ref=$queryByParameterGroupName or @ref=$queryByParameterParameterListGroupName]]"
        mode="messagetype">
        <xsl:comment>
            <xsl:value-of select="@name"/>
            <xsl:text> </xsl:text>
            <xsl:value-of select="name(.)"/>
            <xsl:text> declaration removed</xsl:text>
        </xsl:comment>
    </xsl:template>

    <!-- (messagetype) Remove the RegistrationProcess and MFMI Subject complexType declarations, as they are pulled into the interaction file -->
    <xsl:template
        match="xs:schema/xs:complexType[matches(@name,'^MFMI_MT7007[0-9]{2}(_OPT_OV)?\.Subject$') or matches(@name,'^MFMI_MT7007[0-9]{2}(_OPT_OV)?\.RegistrationProcess$')]"
        mode="messagetype">
        <xsl:comment>
            <xsl:value-of select="@name"/>
            <xsl:text> </xsl:text>
            <xsl:value-of select="name(.)"/>
            <xsl:text> declaration removed</xsl:text>
        </xsl:comment>
    </xsl:template>

    <xsl:template name="outputTransmissionWrapperComplexType">
        <xsl:param name="transmissionWrapperFile"/>
        <xsl:param name="controlActWrapperFile"/>
        <xsl:variable name="transmissionWrapper" select="tokenize(tokenize($transmissionWrapperFile,'/')[last()],'\.')[last()-1]"/>
        <xsl:variable name="controlActWrapper" select="tokenize(tokenize($controlActWrapperFile,'/')[last()],'\.')[last()-1]"/>
        <!--
            Create message type definition. 
            this definition replaces the message type definition in the transmission wrapper.
            Reference to the controlAct is qualified with the interaction name.
        -->
        <xsl:element name="xs:complexType">
            <xsl:attribute name="name">
                <xsl:value-of select="concat($interactionId,'.',$transmissionWrapper,'.Message')"/>
            </xsl:attribute>
            <xsl:for-each select="document($transmissionWrapperFile)/*/xs:complexType[@name=$interactionType]/node()">
                <xsl:choose>
                    <xsl:when test="self::xs:sequence">
                        <xsl:element name="xs:sequence">
                            <xsl:for-each select="./node()">
                                <xsl:choose>
                                    <xsl:when test="self::xs:group[@ref=$controlActGroupName]">
                                        <xsl:element name="xs:element">
                                            <xsl:attribute name="name">
                                                <!-- Get actual element name from ControlAct file, normally 'ControlActProcess' -->
                                                <xsl:value-of
                                                    select="document($controlActWrapperFile)/xs:schema/xs:group[@name=$controlActGroupName]//xs:element/@name"
                                                />
                                            </xsl:attribute>
                                            <xsl:attribute name="type">
                                                <xsl:value-of select="concat($interactionId,'.',$controlActWrapper,'.ControlActProcess')"/>
                                            </xsl:attribute>
                                        </xsl:element>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <xsl:copy-of select="."/>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:for-each>
                        </xsl:element>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:copy-of select="."/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>
        </xsl:element>
    </xsl:template>

    <xsl:template name="outputControlActWrapperComplexType">
        <xsl:param name="transmissionWrapperFile"/>
        <xsl:param name="controlActWrapperFile"/>

        <xsl:variable name="transmissionWrapper" select="tokenize(tokenize($transmissionWrapperFile,'/')[last()],'\.')[last()-1]"/>
        <xsl:variable name="controlActWrapper" select="tokenize(tokenize($controlActWrapperFile,'/')[last()],'\.')[last()-1]"/>

        <xsl:variable name="controlActProcessContents"
            select="document($controlActWrapperFile)/xs:schema/xs:complexType[@name=../xs:group[@name=$controlActGroupName]//xs:element/@type]"/>

        <!-- ControlActProcess complexType with link to Payload in element subject -->
        <xsl:element name="xs:complexType">
            <xsl:attribute name="name">
                <xsl:value-of select="concat($interactionId,'.',$controlActWrapper,'.ControlActProcess')"/>
            </xsl:attribute>
            <xsl:for-each select="$controlActProcessContents/node()">
                <xsl:choose>
                    <xsl:when test="self::xs:sequence">
                        <xsl:element name="xs:sequence">
                            <xsl:for-each select="node()">
                                <xsl:choose>
                                    <xsl:when test="self::xs:element[@name='subject' or @name='queryByParameter']">
                                        <xsl:element name="xs:element">
                                            <xsl:attribute name="name" select="@name"/>
                                            <xsl:attribute name="type" select="concat($interactionId,'.',@type)"/>
                                            <xsl:if test="@minOccurs">
                                                <xsl:attribute name="minOccurs" select="@minOccurs"/>
                                            </xsl:if>
                                            <xsl:if test="@maxOccurs">
                                                <xsl:attribute name="maxOccurs" select="@maxOccurs"/>
                                            </xsl:if>
                                        </xsl:element>
                                    </xsl:when>
                                    <xsl:when test="self::xs:group">
                                        <xsl:apply-templates select="self::node()" mode="#current"/>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <xsl:copy-of select="."/>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:for-each>
                        </xsl:element>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:copy-of select="."/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>
        </xsl:element>

        <!-- ControlActProcess can link to 1 or 2 types of Payload:
            1. element subject will link to a complexType inside the ControlAct file that in turn links to an anynomous group 'Payload'
               Action: get that complexType and replace the <group ref='Payload'/> with the actual contents based on either
               - an empty declaration of Payload inside the interaction file, e.g. for MCCI_IN000002
               - a real declaration of Payload inside the payload file that the interaction file included
            2. <group ref='QueryByParameterPayload'/>
                Action: replace with group contents of QueryByParameterPayload from the payload file that the interaction file included
        -->
        <xsl:if test="$controlActProcessContents/xs:sequence/xs:element[@name='subject']">
            <xsl:variable name="subjectComplexTypeName" select="$controlActProcessContents/xs:sequence/xs:element[@name='subject']/@type"/>
            <xsl:variable name="subjectComplexType" select="document($controlActWrapperFile)/xs:schema/xs:complexType[@name=$subjectComplexTypeName]"/>
            <xsl:element name="xs:complexType">
                <xsl:attribute name="name">
                    <xsl:value-of select="concat($interactionId,'.',$subjectComplexTypeName)"/>
                </xsl:attribute>
                <xsl:for-each select="$subjectComplexType/node()">
                    <xsl:choose>
                        <xsl:when test="self::xs:sequence">
                            <xsl:element name="xs:sequence">
                                <xsl:for-each select="node()">
                                    <xsl:choose>
                                        <xsl:when test="self::xs:element[@name='registrationProcess']">
                                            <xsl:element name="xs:element">
                                                <xsl:attribute name="name" select="$subjectComplexType/xs:sequence/xs:element/@name"/>
                                                <xsl:attribute name="type"
                                                    select="concat($interactionId,'.',$subjectComplexType/xs:sequence/xs:element/@type)"/>
                                                <xsl:if test="$subjectComplexType/xs:sequence/xs:element/@minOccurs">
                                                    <xsl:attribute name="minOccurs" select="$subjectComplexType/xs:sequence/xs:element/@minOccurs"/>
                                                </xsl:if>
                                                <xsl:if test="$subjectComplexType/xs:sequence/xs:element/@maxOccurs">
                                                    <xsl:attribute name="maxOccurs" select="$subjectComplexType/xs:sequence/xs:element/@maxOccurs"/>
                                                </xsl:if>
                                            </xsl:element>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:apply-templates select="." mode="#current"/>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:for-each>
                            </xsl:element>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:copy-of select="."/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:for-each>
            </xsl:element>
        </xsl:if>

        <xsl:if test="document($controlActWrapperFile)/*/xs:complexType[ends-with(@name,'.RegistrationProcess')]">
            <!-- 
               output RegistrationProcess from controlActWrapperFile changing @type subject1 and subject2 to a new complexType inside interaction file
               subject1 is always tied into a complexType ...Subject4, and subject2 is always tied into a complexType ...Subject2, 
               but we'll dynamically get them from RegistrationProcess
            -->
            <xsl:variable name="registrationProcessComplexType"
                select="document($controlActWrapperFile)/*/xs:complexType[ends-with(@name,'.RegistrationProcess')]"/>
            <xsl:element name="xs:complexType">
                <xsl:attribute name="name">
                    <xsl:value-of select="concat($interactionId,'.',$registrationProcessComplexType/@name)"/>
                </xsl:attribute>
                <xsl:element name="xs:sequence">
                    <xsl:for-each select="$registrationProcessComplexType/xs:sequence/node()">
                        <xsl:choose>
                            <xsl:when test="self::xs:element[@name='subject1' or @name='subject2']">
                                <xsl:element name="xs:element">
                                    <xsl:attribute name="name" select="@name"/>
                                    <xsl:attribute name="type" select="concat($interactionId,'.',@type)"/>
                                    <xsl:if test="@minOccurs">
                                        <xsl:attribute name="minOccurs" select="@minOccurs"/>
                                    </xsl:if>
                                    <xsl:if test="@maxOccurs">
                                        <xsl:attribute name="maxOccurs" select="@maxOccurs"/>
                                    </xsl:if>
                                </xsl:element>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:copy-of select="."/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:for-each>
                </xsl:element>
                <xsl:for-each select="$registrationProcessComplexType/xs:sequence/following-sibling::node()">
                    <xsl:copy-of select="."/>
                </xsl:for-each>
            </xsl:element>
            <!-- 
                output new subject1 where group ref='Payload' is rewritten into its actual delcaration based on the current interaction file
                copy anything else as-is
            -->
            <xsl:variable name="subject1ComplexTypeName" select="$registrationProcessComplexType/xs:sequence/xs:element[@name='subject1']/@type"/>
            <xsl:variable name="subject1ComplexType" select="document($controlActWrapperFile)/*/xs:complexType[@name=$subject1ComplexTypeName]"/>
            <xsl:element name="xs:complexType">
                <xsl:attribute name="name">
                    <xsl:value-of select="concat($interactionId,'.',$subject1ComplexType/@name)"/>
                </xsl:attribute>
                <xsl:for-each select="$subject1ComplexType/node()">
                    <xsl:copy>
                        <xsl:apply-templates select="@*|node()" mode="#current"/>
                    </xsl:copy>
                </xsl:for-each>
            </xsl:element>
            <!-- 
                output new subject2 where group ref='Payload' is rewritten into its actual delcaration based on the current interaction file
                copy anything else as-is
            -->
            <xsl:variable name="subject2ComplexTypeName" select="$registrationProcessComplexType/xs:sequence/xs:element[@name='subject2']/@type"/>
            <xsl:variable name="subject2ComplexType" select="document($controlActWrapperFile)/*/xs:complexType[@name=$subject2ComplexTypeName]"/>
            <xsl:element name="xs:complexType">
                <xsl:attribute name="name">
                    <xsl:value-of select="concat($interactionId,'.',$subject2ComplexType/@name)"/>
                </xsl:attribute>
                <xsl:for-each select="$subject2ComplexType/node()">
                    <xsl:copy>
                        <xsl:apply-templates select="@*|node()" mode="#current"/>
                    </xsl:copy>
                </xsl:for-each>
            </xsl:element>
        </xsl:if>

        <xsl:if test="$controlActProcessContents/xs:sequence/xs:element[@name='queryByParameter']">
            <xsl:variable name="queryByParameterComplexTypeName"
                select="$controlActProcessContents/xs:sequence/xs:element[@name='queryByParameter']/@type"/>
            <xsl:variable name="queryByParameterComplexType"
                select="document($controlActWrapperFile)/xs:schema/xs:complexType[@name=$queryByParameterComplexTypeName]"/>
            <xsl:element name="xs:complexType">
                <xsl:attribute name="name">
                    <xsl:value-of select="concat($interactionId,'.',$queryByParameterComplexTypeName)"/>
                </xsl:attribute>
                <xsl:apply-templates select="$queryByParameterComplexType/node()" mode="#current"/>
            </xsl:element>
        </xsl:if>
    </xsl:template>

    <xsl:template name="getTransmissionIncludeFromInteraction">
        <xsl:param name="includeType"/>
        <xsl:for-each select="document($interactionFile)/xs:schema/xs:include">
            <xsl:variable name="includeFileName" select="tokenize(@schemaLocation,'/')[last()]"/>
            <xsl:variable name="includeFileNameFull" select="concat($currentFileDir,@schemaLocation)"/>
            <xsl:variable name="includeFileNameBase" select="tokenize($includeFileName,'\.')[last()-1]"/>
            <xsl:if test="document($includeFileNameFull)/xs:schema/xs:complexType[@name=$includeType]">
                <!--<xsl:value-of select="$includeFileName"/>-->
                <xsl:value-of select="$includeFileNameFull"/>
            </xsl:if>
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="getOtherIncludeFromInteraction">
        <xsl:param name="includeType"/>
        <xsl:for-each select="document($interactionFile)/xs:schema/xs:include">
            <xsl:variable name="includeFileName" select="tokenize(@schemaLocation,'/')[last()]"/>
            <xsl:variable name="includeFileNameFull" select="concat($currentFileDir,@schemaLocation)"/>
            <xsl:variable name="includeFileNameBase" select="tokenize($includeFileName,'\.')[last()-1]"/>
            <xsl:if test="document($includeFileNameFull)/xs:schema/xs:group[@name=$includeType]">
                <xsl:value-of select="$includeFileNameFull"/>
            </xsl:if>
        </xsl:for-each>
    </xsl:template>

    <!-- copy source XML -->
    <xsl:template match="@*|*" mode="#all">
        <xsl:copy>
            <xsl:apply-templates select="@*|node()" mode="#current"/>
        </xsl:copy>
    </xsl:template>

    <!-- Create comment with description of an interaction -->
    <xsl:template name="outputDescriptionComment">
        <xsl:variable name="initialComment">
            <xsl:if test="doc-available($vocabInteractions)">
                <xsl:variable name="displayName"
                    select="document($vocabInteractions)/*/hl7:code[@code=$currentFileNameBase][@codeSystem='2.16.840.1.113883.1.6']/@displayName"/>
                <xsl:if test="string-length($displayName)">
                    <xsl:value-of select="concat(' ' , $currentFileNameBase, ' - ' , $displayName , ' ')"/>
                </xsl:if>
            </xsl:if>
        </xsl:variable>
        <xsl:text>&#10;</xsl:text>
        <xsl:comment>
            <xsl:choose>
                <xsl:when test="string-length($initialComment)>0">
                    <xsl:value-of select="$initialComment"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>Description not available in vocab at generation time</xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:comment>
        <xsl:text>&#10;</xsl:text>
    </xsl:template>
</xsl:stylesheet>
