<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0" xmlns:hl7="urn:hl7-org:v3" exclude-result-prefixes="#all"
    xmlns:File="java:java.io.File" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl" xmlns:xs="http://www.w3.org/2001/XMLSchema">
    <xd:doc scope="stylesheet">
        <xd:desc>
            <xd:p><xd:b>Created on:</xd:b> Jul 7, 2011</xd:p>
            <xd:p><xd:b>Author:</xd:b> gboers</xd:p>
            <xd:p><xd:b>Param:</xd:b> batchType - type of batch file to create. 'sh' for Unix, 'bat' for Windows</xd:p>
            <xd:p><xd:b>Param:</xd:b> inputDir - normally ../../schemas_codeGen or ..\..\schemas_codeGen, the directory to copy the includes from, without a trailing (back)slash</xd:p>
            <xd:p><xd:b>Param:</xd:b> outputDir - normally ../../schemas_codeGen_LSP or ..\..\schemas_codeGen_LSP, the directory to copy the includes to, and where the interaction schemas should already be, without a trailing (back)slash</xd:p>
            <xd:p><xd:b>Purpose:</xd:b> read all interaction XML Schemas from the outputDir that have been transformed in a previous step (see readme.txt in this folder) and follow the includes recursively. The list of includes that is built this way is then written into a text file that may be executed as a batch script as a post-processing step after this transformation finishes.</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:output method="text"/>
    <!-- sh or bat -->
    <xsl:param name="batchType"/>
    <!-- Directory to delete unused schemas from, normally schemas_codeGen_LSP -->
    <xsl:param name="workingDir" required="yes"/>
    <!-- Generated file that contains all includes per file  -->
    <xsl:param name="xmlschemainfo" select="'../resolveIncludes/xmlschemainfo.xml'"/>
    <!-- Convert backslashes to forward slashes -->
    <xsl:variable name="workingdirectory" select="replace(translate($workingDir,'\\','/'),'/$','')"/>
    
    <!-- Expected input:
        <files>
            <file dir="../resolveIncludes/../../schemas" name="COCT_MT000003NL.xsd">
                <uses exists="true" file="datatypes.xsd" full="<fullpath>/coreschemas/datatypes.xsd"/>
                ...
            </file>
            ....
        </files>
    -->
    <xsl:variable name="includeListfull" select="document($xmlschemainfo)"/>
    
    <xsl:variable name="includeList">
        <xsl:call-template name="checkParameters"/>
        <xsl:variable name="tempList">
            <includes>
                <xsl:for-each select="$includeListfull/*/file[matches(@file,'^[A-Z]{4}_IN.*xsd$')]/uses">
                    <xsl:variable name="docUri" select="@full"/>
                    <xsl:variable name="docFileName" select="@file"/>
                    <xsl:if test="not(contains($docUri,'coreschemas'))">
                        <include schemaLocation="{$docUri}" schemaFileName="{$docFileName}"/>
                    </xsl:if>
                </xsl:for-each>
            </includes>
        </xsl:variable>
        <includes>
            <xsl:copy-of select="$tempList/*/include[not(@schemaFileName=preceding-sibling::include/@schemaFileName)]"/>
        </includes>
    </xsl:variable>
        
    <xsl:template match="/">
        <xsl:call-template name="checkParameters"/>
        <xsl:choose>
            <xsl:when test="$batchType='sh'">
                <xsl:text>#!/bin/sh&#xa;</xsl:text>
            </xsl:when>
            <xsl:when test="$batchType='bat'">
                <xsl:text>@echo off&#xd;&#xa;</xsl:text>
            </xsl:when>
        </xsl:choose>
        <xsl:for-each select="$includeListfull//file[not(matches(@file,'^[A-Z]{4}_IN.*xsd$'))]">
            <xsl:sort select="@file"/>
            <xsl:variable name="schemaFileName" select="@file"/>
            <xsl:choose>
                <xsl:when test="$includeList//include[@schemaFileName=$schemaFileName]">
                    <!--xsl:message>File <xsl:value-of select="@name"/> is used: leave as-is</xsl:message-->
                </xsl:when>
                <xsl:otherwise>
                    <!--xsl:message>File <xsl:value-of select="@name"/> is not used: delete</xsl:message-->
                    <xsl:choose>
                        <xsl:when test="$batchType='sh'">
                            <xsl:value-of select="concat('rm ',$workingdirectory,'/',$schemaFileName,'&#xa;')"/>
                        </xsl:when>
                        <xsl:when test="$batchType='bat'">
                            <xsl:value-of select="concat('del /q ',translate($workingdirectory,'/','\\'),'\',$schemaFileName,'&#xd;&#xa;')"/>
                        </xsl:when>
                    </xsl:choose>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
    </xsl:template>
    
    <!-- Check input parameters to this transformation -->
    <xsl:template name="checkParameters">
        <xsl:variable name="errorMessage">
            <xsl:if test="not($batchType='sh' or $batchType='bat')">
                <xsl:value-of select="concat('- Parameter batchType should be &quot;sh&quot; or &quot;bat&quot;. Found &quot;',$batchType,'&quot;&#xa;')"/>
            </xsl:if>
            <xsl:if test="$workingdirectory=''">
                <xsl:value-of select="'- Parameter workingDir is empty&#xa;'"/>
            </xsl:if>
            <xsl:if test="ends-with($workingdirectory,'/')">
                <xsl:value-of select="concat('- Parameter workingDir should not have a trailing (back)slash. Found &quot;',$workingDir,'&quot;&#xa;')"/>
            </xsl:if>
        </xsl:variable>
        <xsl:if test="$errorMessage!=''">
            <xsl:message terminate="yes" select="$errorMessage"/>
        </xsl:if>
    </xsl:template>
</xsl:stylesheet>
