<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0">

    <xsl:output method="text"/>
    <xsl:output method="xml" indent="yes" name="xml"/>

    <!--store variables-->
    <xsl:variable name="wsdlInputPath" select="/system/@wsdlInputPath"/>
    <xsl:variable name="wsdlOutputPath" select="/system/@wsdlOutputPath"/>
    <xsl:variable name="suffix" select="/system/@suffix"/>
    <xsl:variable name="defaultMessage">MCCI_IN000002</xsl:variable>
    <xsl:variable name="schemaPathOrg" select="concat($wsdlOutputPath,'../schemas/')"/>
    <xsl:variable name="schemaPath">../schemas_codeGen/</xsl:variable>


    <!-- This template is for the (primary) source xml, the *_wsdls.xml -->
    <xsl:template match="/system">
        <!-- generate a wsdl document that does not have the htmlOnly flag set -->
        <xsl:for-each select="wsdl[not(@htmlOnly='true')]">
            <xsl:variable name="version" select="if (string-length(./@version)>0) then concat('-', ./@version) else '' "/>
            <xsl:variable name="wsdlInputFile" select="concat($wsdlInputPath, ./@name, $version, '_mep.xml')"/>
            <xsl:variable name="outputFilename" select="concat($wsdlOutputPath, ./@name, $suffix, $version, '.wsdl')"/>
            <xsl:variable name="wsdlInputDoc" select="document($wsdlInputFile)"/>
            <xsl:variable name="wsdlName" select="$wsdlInputDoc/wsdl/@name"/>
            <xsl:variable name="serviceName" select="$wsdlInputDoc/wsdl/service/@name"/>
            <xsl:variable name="version" select="$wsdlInputDoc/wsdl/@version"/>
            <xsl:variable name="versionNeeded" select="string-length($version)>0"/>
            <xsl:variable name="wsdlNameVersion" select="concat($wsdlName, if ($versionNeeded) then concat('-', $version) else '')"/>
            <xsl:variable name="defaultNeeded" select="boolean($wsdlInputDoc//mep[string-length(out/@message)=0 or string-length(in/@message)=0])"/>
            <!-- Output to log  -->
            <xsl:value-of select="concat('Creating ', $outputFilename, ' from ', $wsdlInputFile)"/>
            <xsl:text>&#10;</xsl:text>
            <!-- Output to log  -->

            <xsl:result-document href="{$outputFilename}" format="xml">
                <definitions targetNamespace="urn:hl7-org:v3" xmlns:hl7="urn:hl7-org:v3" xmlns="http://schemas.xmlsoap.org/wsdl/"
                    xmlns:soap="http://schemas.xmlsoap.org/wsdl/soap/" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
                    <xsl:attribute name="name">
                        <xsl:value-of select="concat($wsdlNameVersion, $suffix)"/>
                    </xsl:attribute>
                    <documentation> WSDL implementatie van <xsl:value-of select="concat($wsdlNameVersion, $suffix)"/> </documentation>
                    <types>
                        <!--  Add the include for default message xsd, if needed -->
                        <xsl:if test="$defaultNeeded">
                            <xsd:schema targetNamespace="urn:hl7-org:v3" elementFormDefault="qualified" xmlns:hl7="urn:hl7-org:v3"
                                xmlns:xsd="http://www.w3.org/2001/XMLSchema">
                                <xsl:call-template name="checkIfSchemaExists">
                                    <xsl:with-param name="schemaName" select="concat($defaultMessage, '.xsd')"/>
                                    <xsl:with-param name="outputFilename" select="$outputFilename"/>
                                </xsl:call-template>
                                <xsd:include>
                                    <xsl:attribute name="schemaLocation" select="concat($schemaPath, $defaultMessage, '.xsd')"/>
                                </xsd:include>
                            </xsd:schema>
                        </xsl:if>

                        <!-- Add the includes for message xsd's from the input mep xml, excluding the special composed messages-->
                        <xsl:variable name="uniqueListMessages"
                            select="$wsdlInputDoc//mep/*[string-length(element[1]/@name)=0]/@message[not(.=following::*/@message)]"/>
                        <xsl:for-each select="$uniqueListMessages">
                            <xsd:schema targetNamespace="urn:hl7-org:v3" elementFormDefault="qualified" xmlns:hl7="urn:hl7-org:v3"
                                xmlns:xsd="http://www.w3.org/2001/XMLSchema">
                                <xsl:call-template name="checkIfSchemaExists">
                                    <xsl:with-param name="schemaName" select="concat(., '.xsd')"/>
                                    <xsl:with-param name="outputFilename" select="$outputFilename"/>
                                </xsl:call-template>
                                <xsd:include>
                                    <xsl:attribute name="schemaLocation" select="concat($schemaPath, ., '.xsd')"/>
                                </xsd:include>
                            </xsd:schema>
                        </xsl:for-each>

                        <!--  Add xsd for composed messages  -->
                        <xsl:for-each select="$wsdlInputDoc//mep/*[string-length(element[1]/@name)>0]">
                            <xsd:schema targetNamespace="urn:hl7-org:v3" elementFormDefault="qualified" xmlns:hl7="urn:hl7-org:v3"
                                xmlns:xsd="http://www.w3.org/2001/XMLSchema">
                                <!-- Add includes for the element used in composed messages -->
                                <xsl:variable name="listElementNames" select="./element/@name"/>
                                <xsl:for-each select="$listElementNames">
                                    <xsl:call-template name="checkIfSchemaExists">
                                        <xsl:with-param name="schemaName" select="concat(., '.xsd')"/>
                                        <xsl:with-param name="outputFilename" select="$outputFilename"/>
                                    </xsl:call-template>
                                    <xsd:include>
                                        <xsl:attribute name="schemaLocation" select="concat($schemaPath, ., '.xsd')"/>
                                    </xsd:include>
                                </xsl:for-each>

                                <xsd:element>
                                    <xsl:attribute name="name" select="./@message"/>
                                    <xsd:complexType>
                                        <xsd:choice>
                                            <xsl:for-each select="element">
                                                <xsd:element>
                                                    <xsl:attribute name="ref" select="concat('hl7:',./@name)"/>
                                                </xsd:element>
                                            </xsl:for-each>
                                        </xsd:choice>
                                    </xsd:complexType>
                                </xsd:element>
                            </xsd:schema>
                        </xsl:for-each>
                    </types>

                    <!--  Add the default message, if needed -->
                    <xsl:if test="$defaultNeeded">
                        <message>
                            <xsl:attribute name="name" select="$defaultMessage"/>
                            <part name="body">
                                <xsl:attribute name="element" select="concat('hl7:',$defaultMessage)"/>
                            </part>
                        </message>
                    </xsl:if>

                    <!-- Add the (unique) messages from the input mep xml-->
                    <xsl:variable name="uniqueListMessages" select="$wsdlInputDoc//mep/*/@message[not(.=following::*/@message)]"/>
                    <xsl:for-each select="$uniqueListMessages">
                        <message>
                            <xsl:attribute name="name" select="."/>
                            <part name="body">
                                <xsl:attribute name="element" select="concat('hl7:',.)"/>
                            </part>
                        </message>
                    </xsl:for-each>

                    <!--  Add the port types   -->
                    <portType>
                        <xsl:attribute name="name" select="concat($serviceName, '_PortType')"/>
                        <xsl:for-each select="$wsdlInputDoc//mep">
                            <operation>
                                <xsl:attribute name="name" select="concat($serviceName,if ($versionNeeded) then $version else '', '_', ./@name)"/>
                                <input>
                                    <xsl:attribute name="message"
                                        select="if (string-length(./in/@message)>0) 
                                        then concat('hl7:', in/@message)
                                        else concat('hl7:', $defaultMessage)"
                                    />
                                </input>
                                <output>
                                    <xsl:attribute name="message"
                                        select="if (string-length(./out/@message)>0)
                                        then concat('hl7:', out/@message)
                                        else concat('hl7:', $defaultMessage)"
                                    />
                                </output>
                            </operation>
                        </xsl:for-each>
                    </portType>


                    <!-- Add the binding  -->
                    <binding>
                        <xsl:attribute name="type" select="concat('hl7:', $serviceName, '_PortType')"/>
                        <xsl:attribute name="name" select="concat($serviceName, '_Binding')"/>
                        <soap:binding>
                            <xsl:attribute name="style" select="'document'"/>
                            <xsl:attribute name="transport" select="'http://schemas.xmlsoap.org/soap/http'"/>
                        </soap:binding>
                        <xsl:for-each select="$wsdlInputDoc//mep">
                            <operation>
                                <xsl:attribute name="name" select="concat($serviceName,if ($versionNeeded) then $version else '', '_', ./@name)"/>
                                <soap:operation>
                                    <xsl:attribute name="soapAction" select="concat('urn:hl7-org:v3/', $serviceName,if ($versionNeeded) then $version else '', '_', ./@name)"/>
                                </soap:operation>
                                <input>
                                    <soap:body use="literal"/>
                                </input>
                                <output>
                                    <soap:body use="literal"/>
                                </output>
                            </operation>
                        </xsl:for-each>
                    </binding>

                    <service>
                        <xsl:attribute name="name"
                            select="concat($serviceName, if ($versionNeeded) then
                            concat('_', $version) else '', $suffix, '_Service')"/>
                        <port>
                            <xsl:attribute name="binding" select="concat('hl7:',$serviceName, '_Binding')"/>
                            <xsl:attribute name="name" select="concat($serviceName, '_Port')"/>
                            <xsl:text>&#10;</xsl:text>
                            
                            <xsl:comment> Deze service location URI verwijst niet naar een echt bestaande webservice. </xsl:comment>
                            <xsl:text>&#10;</xsl:text>
                            
                            <soap:address>
                                <xsl:attribute name="location"
                                    select="concat($wsdlInputDoc/wsdl/@location, 
                                    if ($versionNeeded) then
                                    concat($version, '/') else '', 
                                    $serviceName, $suffix)"
                                />
                            </soap:address>
                        </port>
                    </service>
                </definitions>
            </xsl:result-document>
        </xsl:for-each>
        <xsl:for-each select="wsdl[@htmlOnly='true']">
            <xsl:variable name="version" select="if (string-length(./@version)>0) then concat('-', ./@version) else '' "/>
            <xsl:variable name="wsdlInputFile" select="concat($wsdlInputPath, ./@name, $version, '_mep.xml')"/>
            <xsl:variable name="outputFilename" select="concat($wsdlOutputPath, ./@name, $suffix, $version, '.wsdl')"/>
            <!-- Output to log  -->
            <xsl:value-of select="concat('Skipping (htmlOnly flag present): ', $outputFilename)"/>
            <xsl:text>&#10;</xsl:text>
            <!-- Output to log  -->
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="checkIfSchemaExists">
        <xsl:param name="schemaName"/>
        <xsl:param name="outputFilename"/>

        <xsl:if test="not(doc-available(concat($schemaPathOrg,$schemaName)))">
            <xsl:message terminate="no"
                select="concat('[ERROR] Webservice ', $outputFilename, ' verwijst naar niet-bestaand XML Schema ',concat($schemaPathOrg,$schemaName))"/>
            <xsl:comment
                select="concat('[ERROR] Webservice ', $outputFilename, ' verwijst naar niet-bestaand XML Schema ',concat($schemaPathOrg,$schemaName))"/>
            <xsl:text>&#10;</xsl:text>
        </xsl:if>
    </xsl:template>

</xsl:stylesheet>
