package nl.nictiz.xml.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringWriter;
import java.util.Iterator;
import java.util.Vector;

import org.w3c.dom.*;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException; 

/** 
 * <p>Usage: java nl.nictiz.xml.util.AnalyzeSchemas inputDir|inputFile [ outputFile ]</p>
 * <p>Builds xmlschemainfo.xml from one XML schema file, or a full directory of schemas containing:<p>
 * <pre>
 * &lt;files&gt;
 *   &lt;file file="COCT_MT000003NL.xsd" full="-full path-/COCT_MT000003NL.xsd" hl7type="cmet"&gt;
 *     &lt;uses exists="true" file="datatypes.xsd" full="-full path-/datatypes.xsd"/&gt;
 *     &lt;uses exists="true" file="voc.xsd" full="-full path-/voc.xsd"/&gt;
 *     &lt;uses exists="true" file="infrastructureRoot.xsd" full="-full path-/infrastructureRoot.xsd"/&gt;
 *     &lt;uses exists="false" file="COCT_MT150003NL.xsd" full="-full path-/COCT_MT150003NL.xsd"/&gt;
 *     ...
 *     &lt;used-in file="POOR_IN002150NL.xsd" full="-full path-/POOR_IN002150NL.xsd"/&gt;
 *     &lt;used-in file="POOR_MT200999NL.xsd" full="-full path-/POOR_MT200999NL.xsd"/&gt;
 *     ...
 *   &lt;/file&gt;
 *   ...
 * &lt;/files&gt;
 * </pre>
 * 
 * <p>Where:
 * <ul>
 * <li>files   = fixed root element</li>
 * <li>file    = occurs once, if input is a file, or occurs as many times as there are files in the directory</li>
 * <li>uses    = occurs once for each unique include. Each include from the main file is tested for includes and so on. Allows you to build scripts that copy all relevant attached files for a given starting point</li>
 * <li>used-in = since we have all includes for all files, we can also find if and where a given file is included.</li>
 * </ul>
 * </p> 
 * 
 * @author Alexander Henket
 * @version 1.0
 */
public class AnalyzeSchemas {
	
	private static File outputFile = null;
	private static DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();

    /**
     * @param argv Accepts a relative or absolute file or directory as its first mandatory argument, and optionally accepts a file as its second arguments. The first argument is used to determine the list of includes, the second to write the results in, default is to screen. 
     */
    public static void main (String argv []){
	    try {
	    	if (argv.length >= 2 && argv[1] != null) {
	    		outputFile = new File(argv[1]);
	    		System.out.println("Writing result to file: " + outputFile.getCanonicalPath());
	    	}
	    	String string = getIncludes(argv[0]);
	    	if (string != null) {
	    		System.out.println(string);
	    	}
	    }
	    catch (ArrayIndexOutOfBoundsException aiobe) {
	    	System.out.println("Usage: " + AnalyzeSchemas.class.getName() + " inputFileName|inputDirectory [outputFile]");
	    	System.out.println("Where: ");
	    	System.out.println("       inputFileName  = relative or absolute path of an XML Schema file");
	    	System.out.println("       inputDirectory = relative or absolute path of a directory containing XML Schemas");
	    	System.out.println();
	    	System.out.println("       outputFileName = relative or absolute path of an output file, e.g. xmlschemainfo.xml");
	    }
	    catch (SAXParseException err) {
	        System.err.println ("** Parsing error" + ", line " 
	             + err.getLineNumber () + ", uri " + err.getSystemId ());
	        System.err.println(" " + err.getMessage ());
	
	    }
	    catch (Exception e) {
	    	System.err.println(" " + e.getLocalizedMessage());
	    	e.printStackTrace();	
	    }
	    catch (Throwable t) {
	    	System.err.println(" " + t.getLocalizedMessage());
	        t.printStackTrace ();
	    }
    }

    /**
     * This method does the work for main
     * 
     * @param fileName Fully qualified file name to get includes of
     * @return null if output is written to file, or serialized XML string containing the includes
     * @throws ParserConfigurationException
     * @throws SAXException
     * @throws IOException
     * @throws TransformerException
     */
    public static String getIncludes(String fileName) throws ParserConfigurationException, SAXException, IOException, TransformerException {
    	File arg1 = new File(fileName);
    	
    	DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
    	Document doc = docBuilder.newDocument();
    	Element filesElement = doc.createElement("files");
    	doc.appendChild(filesElement);
    	
    	if (arg1.isFile()) {
	    	Vector<String> vectorOfIncludes = new Vector<String>();

	    	Element fileElement = doc.createElement("file");
	    	fileElement.setAttribute("file", arg1.getName());
	    	fileElement.setAttribute("full", arg1.getCanonicalPath());
	    	fileElement.setAttribute("hl7type",getHL7Type(arg1.getName()));
	    	
	    	filesElement.appendChild(fileElement);
	    	
	    	getIncludes(fileName, vectorOfIncludes, doc, fileElement);
	    	
    	} else if (arg1.isDirectory()) {
    		File[] fileList = arg1.listFiles(new XMLSchemaFilter() {});
    		for (int i=0; i<fileList.length; i++) {
    			Vector<String> vectorOfIncludes = new Vector<String>();
    			
    			Element fileElement = doc.createElement("file");
    	    	fileElement.setAttribute("file", fileList[i].getName());
    	    	fileElement.setAttribute("full", fileList[i].getCanonicalPath());
    	    	fileElement.setAttribute("hl7type",getHL7Type(fileList[i].getName()));
    	    	
    	    	filesElement.appendChild(fileElement);
    	    	
    			getIncludes(fileList[i].getAbsolutePath(), vectorOfIncludes, doc, fileElement);
    		}
    	}
    	
    	getReferences(doc);
    	
    	DOMSource domSource = new DOMSource(doc);
    	StreamResult streamResult = null;
    	StringWriter stringWriter = null;
    	if (outputFile != null) {
    		streamResult = new StreamResult(outputFile);
    	} else {
    		stringWriter = new StringWriter();
			streamResult = new StreamResult(stringWriter);
    	}
    	TransformerFactory tf = TransformerFactory.newInstance();
    	Transformer serializer = tf.newTransformer();
    	serializer.setOutputProperty(OutputKeys.ENCODING,"UTF-8");
    	serializer.setOutputProperty(OutputKeys.INDENT,"yes");
    	serializer.transform(domSource, streamResult);
    	if (stringWriter != null) {
    		return stringWriter.toString();
    	} else {
    		return null;
    	}
    }
    
	/**
	 * @param fileName Fully qualified file name to get includes of
	 * @param vectorOfIncludes
	 * @param doc
	 * @param fileElement
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws IOException
	 */
	public static void getIncludes(String fileName,Vector<String> vectorOfIncludes, Document doc, Element fileElement) throws ParserConfigurationException, SAXException, IOException {

	    File file = new File(fileName);
	    try {
	    	DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
	        Document document = docBuilder.parse (file);
	        Vector<Node> vn = new Vector<Node>();
	        
	        Vector<Node> listOfIncludes = nodeListToVector(vn, document.getElementsByTagName("xs:include"));
	        listOfIncludes = nodeListToVector(vn, document.getElementsByTagName("xs:import"));
	        Iterator<Node> iter = listOfIncludes.iterator();
	        
	        while(iter.hasNext()){
	            Node includeNode = iter.next();
	            if (includeNode.getNodeType() == Node.ELEMENT_NODE) {
	                Element includeNodeElement = (Element)includeNode;
	                String schemaLocation = includeNodeElement.getAttribute("schemaLocation");
	                File includeFile = new File(file.getParentFile(),schemaLocation);
	                
	                if (!vectorOfIncludes.contains(includeFile.getName())) {
						//System.out.println(schemaLocation);
		                vectorOfIncludes.add(includeFile.getName());
		                
	                	Element el = doc.createElement("uses");
	                	el.setAttribute("file", includeFile.getName());
	                	el.setAttribute("full", includeFile.getCanonicalPath());
	                	if (includeNodeElement.hasAttribute("namespace")) {
	                		el.setAttribute("namespace",includeNodeElement.getAttribute("namespace"));
	                	}
		                if (includeFile.canRead() && includeFile.isFile()) {
		                	el.setAttribute("exists", "true");
		                	getIncludes(includeFile.getAbsolutePath(), vectorOfIncludes, doc, fileElement);
		                } else {
		                	el.setAttribute("exists", "false");
		                }
		                fileElement.appendChild(el);
	                }
	
	            }
	            //end of if clause
	        }
	        //end of for loop with s var
    	}
    	catch (FileNotFoundException fnfe) {
	    	System.err.println (vectorOfIncludes.lastElement() + " includes/imports an XML Schema that does not exist: " + fileName);
	    	vectorOfIncludes.add(file.getName());
	    }
    }

	/**
     * Get the HL7 type based on file pattern. Currently returns either 'cmet', 'transmissionwrapper', 'controlactwrapper', 'interaction', or 'payload'
     * @param name XML Schema file name
     * @return HL7 type string
     */
    private static String getHL7Type(String name) {
    	String hl7type = "";
    	//System.out.println("getHL7Type,6.2: "+name.substring(5,7));
    	if (name.startsWith("COCT_MT")) {
    		hl7type="cmet";
    	} else if (name.startsWith("POCP_MT")) {
    		hl7type="cmet";
    	} else if (name.startsWith("MCCI_MT")) {
    		hl7type="transmissionwrapper";
    	} else if (name.startsWith("MCAI_MT") || name.startsWith("MFMI_MT") || name.startsWith("QUQI_MT")) {
    		hl7type="controlactwrapper";
    	} else if (name.substring(5,7).equals("IN")) {
    		hl7type="interaction";
    	} else {
    		hl7type="payload";
    	}
        return hl7type;
	}

	/**
     * getReferences walks through all file elements and checks whether or not other file elements have a "uses" element for our current file
     * If so it adds a "used-in" element with @file and @full attributes
     * 
	 * @param doc The XML DOM object containing the files with their includes
	 */
	private static void getReferences(Document doc) {
		// Get all file elements
		NodeList fileList = doc.getElementsByTagName("file");
		
		// Loop through all file elements
		for (int i=0; i<fileList.getLength(); i++) {
			Element currentFileNode = (Element) fileList.item(i);
			String currentFileName = currentFileNode.getAttribute("file");
			//String currentFilePath = currentFileNode.getAttribute("full");
			
			// Loop again through all file elements, finding all other files that "uses" our current file
			for (int j=0; j<fileList.getLength(); j++) {
				// Skip current file
				if (i == j) continue;
				
				Element checkFileNode = (Element) fileList.item(j);
				String checkFileName = checkFileNode.getAttribute("file");
				String checkFilePath = checkFileNode.getAttribute("full");
				
				// Get all uses elements under the current file element
				NodeList usesList = checkFileNode.getElementsByTagName("uses");
				boolean containsCurrentFile = false;
				
				// Loop through all "uses" elements, and mark containsCurrentFile true if matches current file
				//System.out.println("Checking if "+checkFileName+" uses "+currentFileName);
				for (int k=0; k<usesList.getLength(); k++) {
					Element uses = (Element) usesList.item(k);
					if (uses.getAttribute("file").equals(currentFileName)) {
					//	System.out.println("   Yes: "+checkFileName+" uses "+currentFileName);
						containsCurrentFile = true;
					} else {
					//	System.out.println("   No: "+checkFileName+" uses "+uses.getAttribute("file"));
					}
				}
				
				if (containsCurrentFile) {
					Element usedIn = doc.createElement("used-in");
					usedIn.setAttribute("file", checkFileName);
					usedIn.setAttribute("full", checkFilePath);
	                currentFileNode.appendChild(usedIn);
				}
			}
		}
	}
	
	private static Vector<Node> nodeListToVector(Vector<Node> vn, NodeList nl) {
		for (int s=0 ; s<nl.getLength() ; s++) {
			vn.add(nl.item(s));
		}
		return vn;
	}
}